from .legal_consent_view import LegalConsentView
from .terms_of_use_view import TermsOfUseView
from .privacy_notice_view import PrivacyNoticeView

__all__ = [
    "LegalConsentView",
    "TermsOfUseView",
    "PrivacyNoticeView",
]
